function MyScroll() {
  this.object = null;
  this.content = null;
  this.isTouchDevice = false;
  this.start = false;
  this.scrollbar = null;
  this.scroller = null;
  this.startX = 0;
  this.startY = 0;
  this.curX = 0;
  this.curY = 0;
  this.lastMove = null;
}

MyScroll.prototype = {
  touchStart: function (event) {
    if (event.type === 'touchstart') { //touch event
      // check that only one finger is touching the screen
      if (event.touches.length === 1) {
        // get the coordinates of the touch
        this.start = true;
        this.startX = event.touches[0].pageX;
        this.startY = event.touches[0].pageY;
      } else {
        // more than one finger touched so cancel
        this.touchCancel(event);
      }
    } else {
      if (event.button === 0) { // mouse event
        this.start = true;
        this.startX = event.pageX;
        this.startY = event.pageY;
      } else { // old IE mouse event
        this.start = true;
        this.startX = event.clientX + document.body.scrollLeft;
        this.startY = event.clientY + document.body.scrollTop;
      }
    }
  },
  touchMove: function (event) {
    if (this.start) {
      var bScroll = false;
      if (event.type === 'touchmove') { // touch event
        // check that only one finger is touching the screen
        if (event.touches.length === 1) {
          this.curX = event.touches[0].pageX;
          this.curY = event.touches[0].pageY;
          event.preventDefault();
          bScroll = true;
        } else {
          // more than one finger touched so cancel
          this.touchCancel(event);
        }
      } else if (event.pageX) { // mouse event
        this.curX = event.pageX;
        this.curY = event.pageY;
        bScroll = true;
      } else { // old IE mouse event
        this.curX = event.clientX + document.body.scrollLeft;
        this.curY = event.clientY + document.body.scrollTop;
        bScroll = true;
      }
      if (bScroll) {
        var X = (this.startX-this.curX);
        var Y = (this.startY-this.curY);
        this.startX = this.curX;
        this.startY = this.curY;
        this.scroll(X, Y);
      }
    }
  },
  touchEnd: function (event) {
    if (this.start) {
      //this.touchMove(event);
      this.transition();
      this.start = false;
    }
    this.startX = 0;
    this.startY = 0;
    this.curX = 0;
    this.curY = 0;
  },
  wheel: function(event) { // mouse wheel event
    var X = 0, Y = 0;
    if (event.wheelDeltaX !== undefined) {
      X = event.wheelDeltaX / 6;
      Y = event.wheelDeltaY / 6;
    } else if(event.wheelDelta !== undefined) {
      X = Y = event.wheelDelta / 6;
    } else if (event.detail !== undefined) {
      X = Y = -event.detail * 6;
    } else {
      return;
    }
    this.scroll(0, Y);
  },
  transition: function() {
    this.content.style.top = (parseInt(this.content.style.top, 10) + this.lastMove*10) + 'px';
    this.content.style.transition = "top 1s ease-out";
  },
  transitionend: function(event) {
    if (parseInt(this.content.style.top, 10) > 0) {
      this.content.style.top = '0px';
    }
    if (Math.abs(parseInt(this.content.style.top, 10)) > (parseInt(this.content.offsetHeight, 10) - parseInt(this.object.offsetHeight, 10))) {
      this.content.style.top = - (parseInt(this.content.offsetHeight, 10) - parseInt(this.object.offsetHeight, 10));
    }
    this.content.style.transition = "none";
  },
  scroll: function(X, Y) {
    if (this.content.offsetWidth > this.object.offsetWidth) {
      this.content.style.left = parseInt(this.content.style.left, 10) - X + 'px';
    }
    if ((this.content.offsetHeight > this.object.offsetHeight) &&
        (((Math.abs(parseInt(this.content.style.top, 10)) < (parseInt(this.content.offsetHeight, 10) - parseInt(this.object.offsetHeight, 10))) && Y>0) ||
         ((parseInt(this.content.style.top, 10) < 0) && Y<0))) {
      this.lastMove = parseInt(this.content.style.top, 10);
      this.content.style.top = parseInt(this.content.style.top, 10) - Y + 'px';
      this.lastMove = parseInt(this.content.style.top, 10) - this.lastMove;
    }
    this.scroller.style.top = Math.round((this.scrollbar.offsetHeight ? this.scrollbar.offsetHeight : this.object.offsetHeight) * (Math.abs(parseInt(this.content.style.top, 10)) / parseInt(this.content.offsetHeight, 10))) + "px";
  },
  resize: function() {
    var offsetHeight = (this.scrollbar.offsetHeight ? this.scrollbar.offsetHeight : this.object.offsetHeight);
    this.content.style.width = this.object.offsetWidth-10 + "px";
    this.scroller.style.height = Math.min(Math.max(Math.round(offsetHeight * (this.object.offsetHeight / this.content.offsetHeight)), 8), offsetHeight-6) + "px";
    this.scroller.style.top = Math.round(offsetHeight * (Math.abs(parseInt(this.content.style.top, 10)) / parseInt(this.content.offsetHeight, 10))) + "px";
    if (this.content.offsetHeight <= this.object.offsetHeight) {
      this.scroller.style.display = "none";
    } else {
      this.scroller.style.display = "block";
    }
  },
  handle: function(event) {
    event || (event = window.event);
    switch (event.type) {
      case 'mousedown':
      case 'touchstart': this.touchStart(event); break;
      case 'mousemove':
      case 'touchmove': this.touchMove(event); break;
      case 'mouseup':
      case 'touchend':
      case 'mouseout':
      case 'touchcancel': this.touchEnd(event); break;
      case 'mousewheel':
      case 'DOMMouseScroll': this.wheel(event); break;
      case 'transitionend':
      case 'webkitTransitionEnd':
      case 'oTransitionEnd':
      case 'MSTransitionEnd': this.transitionend(event); break;
      case 'onorientationchange':
      case 'resize': this.resize(); break;
    }
    if (event.preventDefault) {
      event.preventDefault();
    }
  },
  bind: function(ev, el) {
    var self = this;
    if (window.addEventListener !== undefined) { // W3C
      var handler = (el || window).addEventListener(ev,
        function(event){
          self.handle(event);
        });
    } else if (window.attachEvent) { // old IE
      (el || window).attachEvent('on'+ev, function(event){self.handle(event);});
    } else { // fallback
      (el || window)['on'+ev] = function(event){self.handle(event);};
    }
  },

  unbind: function (ev, el) {
    if (window.removeEventListener !== undefined) { // W3C
      (el || this.scroller).removeEventListener(ev, this, !!bubble);
    }
  },
  init: function(scroll) {
    this.object = scroll;
    if (scroll.style.position === "") {
      scroll.style.position = "relative";
    }
    scroll.style.overflow = "hidden";
    this.content = document.createElement("div");
    while (scroll.hasChildNodes()) {
      this.content.appendChild(scroll.removeChild(scroll.firstChild));
    }
    scroll.appendChild(this.content);
    this.content.style.position = "absolute";
    this.content.style.left = '0px';
    this.content.style.top = '0px';
    //create scrollbar
    this.scrollbar = document.createElement("div");
    this.scrollbar.style.position = "absolute";
    this.scrollbar.style.width = "7px";
    this.scrollbar.style.bottom = "2px";
    this.scrollbar.style.top = "2px";
    this.scrollbar.style.right = "1px";
    this.object.appendChild(this.scrollbar);
    this.scroller = document.createElement("div");
    this.scroller.style.position = "absolute";
    this.scroller.style.backgroundColor = "#7A7A7A";
    this.scroller.style.width = "100%";
    this.scroller.style.borderRadius = "3px";
    this.scroller.style.opacity = 0.5;
    this.scrollbar.appendChild(this.scroller);
    this.resize();
    this.isTouchDevice = (window.ontouchstart !== undefined);
    if (this.isTouchDevice) {
      this.bind('touchstart', scroll);
      this.bind('touchmove', scroll);
      this.bind('touchend', scroll);
      this.bind('touchcancel', scroll);
    } else {
      this.bind('mousedown', scroll);
      this.bind('mousemove', scroll);
      this.bind('mouseup', scroll);
      this.bind('mouseout', scroll);
      this.bind('mouseover', scroll);
      this.bind('mousewheel');
      this.bind('DOMMouseScroll');
    }
    var style = document.documentElement.style;
    if (style.transition !== undefined) {
      this.bind('transitionend');
    }
    this.bind('resize');
  }
};

var i, els = document.querySelectorAll('div.scroll');
for(i=0; i<els.length; i++) {
  els[i].style.border = '1px solid #777';
  els[i].style.height = '200px';
  els[i].style.position = "relative";
  els[i].style.overflowY = 'auto';
  // change scroll direction
  /*var scroll = new MyScroll();
  scroll.init(els[i]);*/
}
